/*
Copyright (c) 2003-2010, CKSource - Frederico Knabben. All rights reserved.
For licensing, see LICENSE.html or http://ckeditor.com/license
*/

define("ckeditor/plugins/eppagebreak/plugin", [
	"jquery",
	"ckeditor/ckeditor",
	"$dict!ep/dict"
], function ($, CKEDITOR, epDict) {

	/*
	 * @dictionary		ep.dict
	 *
	 * @translation		{PageBreak}
	 */

	// register some global properties and methods
	CKEDITOR.eppagebreakClass = 'ckePageBreak';

	CKEDITOR.eppagebreakText = ' more ';

	CKEDITOR.extend = function(targetObj, sourceObj){
		var defaultObj = {
				getAttribute: function(){
					return new String();
				},
				attributes: {
					align: new String()
				}
			};

		// set a valid object
		sourceObj = sourceObj || defaultObj;
		// extend target object
		for(var key in sourceObj){
			targetObj[key] = sourceObj[key];
		}
	};

	CKEDITOR.plugins.add('eppagebreak', {
		requires: ['fakeobjects'],

		init: function(ed){
			var baseUrl = CKEDITOR.getUrl(this.path),

				tPageBreak = epDict.translate('PageBreak')

				checkMultipleElement = function(tag, className){
					var elems = ed.document.getElementsByTag(tag);

					// loop over all elements
					for(var i=0, iLength=elems.count(); i<iLength; i++){
						var elem = elems.getItem(i);

						// if already a more-element exists
						if(elem.hasClass(className)){
							elem.remove();
						}
					}

					// no PageBreak exists => add PageBreak
					addPageBreak(CKEDITOR.eppagebreakText);
				},

				addPageBreak = function(commentText){
					// extend comment prototype for dom object
					var proto = CKEDITOR.dom.comment.prototype;
					if(!proto.getAttribute) {
						CKEDITOR.extend(proto);
					}

					var comment = new CKEDITOR.dom.comment(commentText),
						fakedElem = ed.createFakeElement(comment, CKEDITOR.eppagebreakClass, 'hr'),
						domRange = ed.getSelection().getRanges()[0];

					// add faked element
					domRange.insertNode(fakedElem);

					// search new position for cursor
					var followingElement = fakedElem;
					do{
						followingElement = followingElement.getNext();
					}while(followingElement && !domRange.moveToElementEditStart(followingElement));

					// set cursor
					domRange.select();
				};

			// button for toolbar
			ed.ui.addButton('eppagebreak', {
				label: tPageBreak,
				command: 'eppagebreak',
				icon: baseUrl + 'images/pagebreak.png'
			});

			// add PageBreak command
			ed.addCommand('eppagebreak', {
				exec: function(){
					// handle multiple PageBreak elements
					checkMultipleElement('img', CKEDITOR.eppagebreakClass);
				}
			});

		},

		afterInit: function(ed){
			// modify dataProcessor to replace comments by fake elements
			ed.dataProcessor.dataFilter.addRules({
				comment: function(commentText){
					var retVal = commentText,
						proto = CKEDITOR.htmlParser.comment.prototype;

					// extend comment prototype for htmlParser
					if(!proto.getAttribute){
						CKEDITOR.extend(proto);
					}

					// return fake element if PageBreak-text was overhanded, otherwise overhanded text is returned in its original state
					return (commentText == CKEDITOR.eppagebreakText)? ed.createFakeParserElement(new CKEDITOR.htmlParser.comment(commentText), CKEDITOR.eppagebreakClass, 'hr') : commentText;
				}
			});
		}
	});

	return CKEDITOR;

});
