(function () {
    'use strict';
    var scribePluginPlaceholder = function (toolbarNode, options) {
       var dropdown = toolbarNode.querySelector(options.selector);

        /**
         * toggleDropdownHandler for toggling the dropdown menu
         *
         * @param {Event} event event object
         */
        var toggleDropdownHandler = function () {
            //gSelection = new scribe.api.Selection();
            dropdown.classList.toggle('open');
            if (dropdown.classList.contains('open')) {
                document.addEventListener('click', cancelHandler);
            } else {
                document.removeEventListener('click', cancelHandler);
            }
        };

        /**
         * cancelHandler for closing dropdown when clicked
         * somewhere outside the toolbar
         *
         * @param {Event} event event object
         */

        var cancelHandler = function (event) {
            if (!dropdown.contains(event.target)) {
                toggleDropdownHandler();
            }
        };

        return function (scribe) {
            // if a placeholder dropdown exists
            if (!dropdown) {
                return;
            }

            // Create own simulated command for the dropdown-toggle
            var commandNodes = dropdown.querySelectorAll('[data-command-name]'),

                // Replace selected text in scribe text area with valid html
                replaceSelectionWithHtml = function (scribe, html) {
                    var selection = new scribe.api.Selection(),
                        range = selection.selection.getRangeAt(0);

                    range.deleteContents();
                    scribe.insertHTML(html);
                },

                // Register commands with their respective command action
                registerCommands = function (scribe, commandName, placeholder) {
                    var command = new scribe.api.Command(commandName);

                    command.execute = function () {
                        replaceSelectionWithHtml(scribe, placeholder);
                    };

                    command.queryEnabled = function () {
                         return true;
                    };

                    return command;
                };

            // Get scribe-commands for the dropdown with their respective command action
            var commands = [].map.call(commandNodes, function (node) {
                return {
                    command: (node.dataset.commandName === "dropdownToggle") ? scribe.getCommand(node.dataset.commandName) : registerCommands(scribe, node.dataset.commandName, node.dataset.placeholderValue),
                    title: node.textContent,
                    name: node.dataset.commandName
                };
            });

            // Add EventListener to dropdown field to toggle
            dropdown.addEventListener('click', toggleDropdownHandler);

            // Register commands on scribe object
            commands.forEach(function (command, index) {
                scribe.commands[command.name] = command.command;
            });
        };
    };

    // Module system magic dance
    if (typeof module !== 'undefined' && module && typeof module.exports === "object") {
        module.exports = scribePluginPlaceholder;
    } else if (typeof define === 'function' && typeof define.amd === 'object') {
        define(function () {
            return scribePluginPlaceholder;
        });
    } else {
        window.scribePluginPlaceholder = scribePluginPlaceholder;
    }
}());