/*
 * @copyright        © Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module            de_epages.presentation.inc.mainMenu
 */

 /*jslint browser: true, regexp: true*/
 /*global define*/

 define('de_epages/presentation/inc/mainmenu-unity', [
     'jquery',
     '$tmpl!./mainmenu-unity'
 ], function ($, tmplMainmenu) {

     'use strict';

     var statusClassesMap = {
             DISABLED: 'Disabled',
             FEATURE_DISABLED: 'FeatureDisabled',
             HIDDEN: 'Hidden',
             SELECTED: 'Selected ui-hover'
         },

         // Helper function to toggle icons on menu items
         toggleIcon = function($el) {
             $el
                 .find('.fa-angle-right, .fa-angle-down')
                 .toggleClass('fa-angle-right fa-angle-down');
         },

         // Used for 'delayed' *:hover* later on.
         liWithChild;

     return {
        buildMenu: function (selector, updates) {
            var $node = $(selector),
                menu = $node.data('menu'),
                tmplHelper = {
                    'updates' : updates,
                    // Setup class attribute of menu item's *li* node.
                    'classAttr' : function () {
                        var self = this,
                            data = self.data,
                            classes = [];
                        if (data.menualias) {
                            classes.push(data.menualias.toLowerCase().replace(/[^a-z0-9-_]/g, ''));
                        }
                        if (data.children && data.children.length) {
                            classes.push('hasChildren');
                        }
                        if (self.followedBySeparator) {
                            classes.push('Separator');
                        }
                        if (statusClassesMap[data.status]) {
                            classes.push(statusClassesMap[data.status]);
                        }
                        return classes.length < 1 ? '' : 'class="' + classes.join(' ') + '"';
                    },
                    // Update menu item.
                    'setup' : function () {
                        var self = this,
                            data = self.data,
                            update = self.updates[data.menuId];
                        if (update) {
                            $.extend(data, update);
                        }
                        return '';
                    }
                };

            // Render template (which also handles updating menu items),
            // append to *node* and assign *onClick* handler, if so desired.
            tmplMainmenu(menu, tmplHelper)
                .appendTo($node)
                .find('.ep-menu-click').each(function () {
                    var $this = $(this);
                    $this.on('click', $this.tmplItem('data').onClick);
                });
            // Simulate a delayed *:hover* status with *jQuery.delay* and the *ui-hover* class.
            liWithChild = $node.find('li.hasChildren');
            liWithChild
                .on('click', '> div', function (event) {
                    event.preventDefault();
                    var $this = $(this),
                        lis = liWithChild
                            .filter('.ui-hover')
                            .not(this)
                            .not(function () {
                                return $.contains(this, event.target);
                            })
                            .clearQueue()
                            .removeClass('ui-hover');
                    toggleIcon(lis.find('> div'));
                    // Add *ui-hover* class to *this*.
                    $this.parent().toggleClass('ui-hover');
                    $this.clearQueue();
                    toggleIcon($this);
                });
        },

        getMenuObject: function (selector) {
            return $(selector).data('menu');
        }
    };
 });
